package com.tourism.webadmin.back.dto;

import com.tourism.common.core.validator.UpdateGroup;

import io.swagger.v3.oas.annotations.media.Schema;
import lombok.Data;

import jakarta.validation.constraints.*;

/**
 * 用户浏览记录表Dto对象。
 *
 * @author 吃饭睡觉
 * @date 2024-09-06
 */
@Schema(description = "用户浏览记录表Dto对象")
@Data
public class TourBrowseRecordsDto {

    /**
     * id。
     * NOTE: 可支持等于操作符的列表数据过滤。
     */
    @Schema(description = "id。可支持等于操作符的列表数据过滤。", requiredMode = Schema.RequiredMode.REQUIRED)
    @NotNull(message = "数据验证失败，id不能为空！", groups = {UpdateGroup.class})
    private Long id;

    /**
     * 浏览者id。
     * NOTE: 可支持等于操作符的列表数据过滤。
     */
    @Schema(description = "浏览者id。可支持等于操作符的列表数据过滤。", requiredMode = Schema.RequiredMode.REQUIRED)
    @NotNull(message = "数据验证失败，浏览者id不能为空！")
    private Long visitorId;

    /**
     * 原创游记作者id。
     * NOTE: 可支持等于操作符的列表数据过滤。
     */
    @Schema(description = "原创游记作者id。可支持等于操作符的列表数据过滤。", requiredMode = Schema.RequiredMode.REQUIRED)
    @NotNull(message = "数据验证失败，原创游记作者id不能为空！")
    private Long writerId;

    /**
     * 游记、项目、签证id。
     * NOTE: 可支持等于操作符的列表数据过滤。
     */
    @Schema(description = "游记、项目、签证id。可支持等于操作符的列表数据过滤。", requiredMode = Schema.RequiredMode.REQUIRED)
    @NotNull(message = "数据验证失败，游记、项目、签证id不能为空！")
    private Long itemId;

    /**
     * 浏览项目类型id  1游记 2项目 3签证。
     * NOTE: 可支持等于操作符的列表数据过滤。
     */
    @Schema(description = "浏览项目类型id  1游记 2项目 3签证。可支持等于操作符的列表数据过滤。", requiredMode = Schema.RequiredMode.REQUIRED)
    @NotNull(message = "数据验证失败，浏览项目类型id  1游记 2项目 3签证不能为空！")
    private Integer itemTypeId;
}
