package com.tourism.webadmin.back.service.impl;

import cn.hutool.core.collection.CollUtil;
import com.alibaba.fastjson.JSONObject;
import com.alibaba.fastjson.TypeReference;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.tourism.webadmin.back.model.constant.IsTrue;
import com.tourism.webadmin.back.service.*;
import com.tourism.webadmin.back.dao.*;
import com.tourism.webadmin.back.model.*;
import com.tourism.webadmin.config.DataSourceType;
import com.tourism.webadmin.upms.service.SysUserService;
import com.tourism.common.core.annotation.MyDataSource;
import com.tourism.common.core.base.dao.BaseDaoMapper;
import com.tourism.common.core.constant.GlobalDeletedFlag;
import com.tourism.common.core.object.MyRelationParam;
import com.tourism.common.core.object.CallResult;
import com.tourism.common.core.base.service.BaseService;
import com.tourism.common.core.util.MyModelUtil;
import com.tourism.common.sequence.wrapper.IdGeneratorWrapper;
import com.tourism.common.dict.service.GlobalDictService;
import com.github.pagehelper.Page;
import lombok.extern.slf4j.Slf4j;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.*;

/**
 * 聊天群聊管理数据操作服务类。
 *
 * @author 吃饭睡觉
 * @date 2024-09-06
 */
@Slf4j
@MyDataSource(DataSourceType.MAIN)
@Service("tourImGroupService")
public class TourImGroupServiceImpl extends BaseService<TourImGroup, Long> implements TourImGroupService {

    @Autowired
    private IdGeneratorWrapper idGenerator;
    @Autowired
    private TourImGroupMapper tourImGroupMapper;
    @Autowired
    private TourImGroupTypeService tourImGroupTypeService;
    @Autowired
    private SysUserService sysUserService;
    @Autowired
    private TourImGroupInvitationService tourImGroupInvitationService;
    @Autowired
    private TourImMemberService tourImMemberService;
    @Autowired
    private TourImMessageService tourImMessageService;
    @Autowired
    private GlobalDictService globalDictService;

    /**
     * 返回当前Service的主表Mapper对象。
     *
     * @return 主表Mapper对象。
     */
    @Override
    protected BaseDaoMapper<TourImGroup> mapper() {
        return tourImGroupMapper;
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public TourImGroup saveNew(TourImGroup tourImGroup) {
        tourImGroupMapper.insert(this.buildDefaultValue(tourImGroup));
        return tourImGroup;
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public void saveNewBatch(List<TourImGroup> tourImGroupList) {
        if (CollUtil.isNotEmpty(tourImGroupList)) {
            tourImGroupList.forEach(this::buildDefaultValue);
            tourImGroupMapper.insertList(tourImGroupList);
        }
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public void saveNewBatch(List<TourImGroup> tourImGroupList, int batchSize) {
        if (CollUtil.isEmpty(tourImGroupList)) {
            return;
        }
        if (batchSize <= 0) {
            batchSize = 10000;
        }
        int start = 0;
        do {
            int end = Math.min(tourImGroupList.size(), start + batchSize);
            List<TourImGroup> subList = tourImGroupList.subList(start, end);
            // 如果数据量过大，同时当前表中存在createTime或updateTime等字段，可以考虑在外部创建一次 new Date()，
            // 然后传入buildDefaultValue，这样可以减少对象的创建次数，降低GC，提升效率。橙单之所以没有这样生成，是因为
            // 有些业务场景下需要按照这两个日期字段排序，因此我们只是在这里给出优化建议。
            subList.forEach(this::buildDefaultValue);
            tourImGroupMapper.insertList(subList);
            if (end == tourImGroupList.size()) {
                break;
            }
            start += batchSize;
        } while (true);
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public TourImGroup saveNewWithRelation(TourImGroup tourImGroup, JSONObject relationData) {
        this.saveNew(tourImGroup);
        this.saveOrUpdateRelationData(tourImGroup, relationData);
        return tourImGroup;
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public boolean update(TourImGroup tourImGroup, TourImGroup originalTourImGroup) {
        MyModelUtil.fillCommonsForUpdate(tourImGroup, originalTourImGroup);
        // 这里重点提示，在执行主表数据更新之前，如果有哪些字段不支持修改操作，请用原有数据对象字段替换当前数据字段。
        UpdateWrapper<TourImGroup> uw = this.createUpdateQueryForNullValue(tourImGroup, tourImGroup.getId());
        return tourImGroupMapper.update(tourImGroup, uw) == 1;
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public boolean updateWithRelation(
            TourImGroup tourImGroup, TourImGroup originalTourImGroup, JSONObject relationData) {
        // tourImGroup 为空的时候，无需修改主表数据。
        if (tourImGroup != null && !this.update(tourImGroup, originalTourImGroup)) {
            return false;
        }
        this.saveOrUpdateRelationData(originalTourImGroup, relationData);
        return true;
    }

    private void saveOrUpdateRelationData(TourImGroup tourImGroup, JSONObject relationData) {
        List<TourImGroupInvitation> tourImGroupInvitationList =
                relationData.getObject("tourImGroupInvitationList", new TypeReference<List<TourImGroupInvitation>>() {});
        // 对于一对多更新，分为以下三步：
        // 1. 在关联从表中，删除掉与主表字段关联，但是又没有出现在本地更新中的数据。我们将这些数据视为需要删除的数据。
        // 2. 在本次更新数据列表中，如果从表的对象没有主键Id，我们视为新数据，可以批量插入。
        // 3. 在本次更新数据列表中，如果从表的对象存在主键Id，我们视为已有数据，逐条更新。
        if (tourImGroupInvitationList != null) {
            tourImGroupInvitationService.updateBatchByGroupId(tourImGroup.getId(), tourImGroupInvitationList);
        }
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public boolean remove(Long id) {
        if (tourImGroupMapper.deleteById(id) == 0) {
            return false;
        }
        tourImGroupInvitationService.removeByGroupId(id);
        return true;
    }

    @Override
    public List<TourImGroup> getTourImGroupList(TourImGroup filter, String orderBy) {
        return tourImGroupMapper.getTourImGroupList(filter, orderBy);
    }

    @Override
    public List<TourImGroup> getTourImGroupListWithRelation(TourImGroup filter, TourImGroupInvitation tourImGroupInvitationFilter, TourImMember tourImMemberFilter, TourImMessage tourImMessageFilter, String orderBy) {
        List<TourImGroup> resultList =
                tourImGroupMapper.getTourImGroupListEx(filter, tourImGroupInvitationFilter, tourImMemberFilter, tourImMessageFilter, orderBy);
        // 在缺省生成的代码中，如果查询结果resultList不是Page对象，说明没有分页，那么就很可能是数据导出接口调用了当前方法。
        // 为了避免一次性的大量数据关联，规避因此而造成的系统运行性能冲击，这里手动进行了分批次读取，开发者可按需修改该值。
        int batchSize = resultList instanceof Page ? 0 : 1000;
        this.buildRelationForDataList(resultList, MyRelationParam.normal(), batchSize);
        return resultList;
    }

    @Override
    public CallResult verifyRelatedDataBeforeDelete(TourImGroup tourImGroup) {
        TourImMember tourImMember = new TourImMember();
        tourImMember.setGroupId(tourImGroup.getId());
        if (tourImMemberService.getCountByFilter(tourImMember) > 0) {
            // NOTE: 可以根据需求修改下面方括号中的提示信息。
            return CallResult.error("数据验证失败，[TourImMember] 存在关联数据！");
        }
        TourImMessage tourImMessage = new TourImMessage();
        tourImMessage.setGroupId(tourImGroup.getId());
        if (tourImMessageService.getCountByFilter(tourImMessage) > 0) {
            // NOTE: 可以根据需求修改下面方括号中的提示信息。
            return CallResult.error("数据验证失败，[TourImMessage] 存在关联数据！");
        }
        return CallResult.ok();
    }

    @Override
    public CallResult verifyImportList(List<TourImGroup> dataList, Set<String> ignoreFieldSet) {
        CallResult callResult;
        if (!CollUtil.contains(ignoreFieldSet, "isPublic")) {
            callResult = verifyImportForConstDict(dataList, "isPublicDictMap", TourImGroup::getIsPublic);
            if (!callResult.isSuccess()) {
                return callResult;
            }
        }
        if (!CollUtil.contains(ignoreFieldSet, "isNeedConfirm")) {
            callResult = verifyImportForConstDict(dataList, "isNeedConfirmDictMap", TourImGroup::getIsNeedConfirm);
            if (!callResult.isSuccess()) {
                return callResult;
            }
        }
        if (!CollUtil.contains(ignoreFieldSet, "bannedStatus")) {
            callResult = verifyImportForConstDict(dataList, "bannedStatusDictMap", TourImGroup::getBannedStatus);
            if (!callResult.isSuccess()) {
                return callResult;
            }
        }
        if (!CollUtil.contains(ignoreFieldSet, "noticeType")) {
            callResult = verifyImportForGlobalDict(dataList, "noticeTypeDictMap", TourImGroup::getNoticeType);
            if (!callResult.isSuccess()) {
                return callResult;
            }
        }
        if (!CollUtil.contains(ignoreFieldSet, "belongTypeId")) {
            callResult = verifyImportForDatasourceDict(dataList, "belongTypeIdDictMap", TourImGroup::getBelongTypeId);
            if (!callResult.isSuccess()) {
                return callResult;
            }
        }
        if (!CollUtil.contains(ignoreFieldSet, "leaderId")) {
            callResult = verifyImportForDatasourceDict(dataList, "leaderIdDictMap", TourImGroup::getLeaderId);
            if (!callResult.isSuccess()) {
                return callResult;
            }
        }
        return CallResult.ok();
    }

    @Override
    public CallResult verifyRelatedData(TourImGroup tourImGroup, TourImGroup originalTourImGroup) {
        String errorMessageFormat = "数据验证失败，关联的%s并不存在，请刷新后重试！";
        //这里是基于字典的验证。
        if (this.needToVerify(tourImGroup, originalTourImGroup, TourImGroup::getBelongTypeId)
                && !tourImGroupTypeService.existId(tourImGroup.getBelongTypeId())) {
            return CallResult.error(String.format(errorMessageFormat, "群聊所属类型id"));
        }
        //这里是基于字典的验证。
        if (this.needToVerify(tourImGroup, originalTourImGroup, TourImGroup::getLeaderId)
                && !sysUserService.existId(tourImGroup.getLeaderId())) {
            return CallResult.error(String.format(errorMessageFormat, "群主id"));
        }
        //这里是基于字典的验证。
        if (this.needToVerify(tourImGroup, originalTourImGroup, TourImGroup::getNoticeType)
                && !globalDictService.existDictItemFromCache("TourImSingleOrGroup", tourImGroup.getNoticeType())) {
            return CallResult.error(String.format(errorMessageFormat, "通知分类 0单聊 1群聊"));
        }
        return CallResult.ok();
    }

    private TourImGroup buildDefaultValue(TourImGroup tourImGroup) {
        if (tourImGroup.getId() == null) {
            tourImGroup.setId(idGenerator.nextLongId());
        }
        MyModelUtil.fillCommonsForInsert(tourImGroup);
        tourImGroup.setDataState(GlobalDeletedFlag.NORMAL);
        MyModelUtil.setDefaultValue(tourImGroup, "description", "");
        MyModelUtil.setDefaultValue(tourImGroup, "hotValue", 0);
        MyModelUtil.setDefaultValue(tourImGroup, "isPublic", IsTrue.TRUE);
        MyModelUtil.setDefaultValue(tourImGroup, "bannedStatus", IsTrue.FALSE);
        return tourImGroup;
    }
}
